// File:  RdbStats.cc

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rdbstats
//
// rdbstats is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rdbstats is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "RdbStats.h"

RdbStats::~RdbStats( ) {

  //  if ( output_stats ) {
    //    for ( int ii = 0; output_stats[ ii ] != NULL; ii++ )
    //      delete output_stats[ ii ];
  //    delete [] output_stats;
  //    output_stats = NULL;
  //  }

  // Note, input_column is not to be delete since RDB++ lib will do it.

}


RdbStats::RdbStats( RDB& irdbtable, const std::string& name )
  : input_column( NULL ), output_stats( ), num_n( 0 ), colname( name ) {
    init( );
    input_column = (RDBColumn*) irdbtable.getColumn( name );
}

int RdbStats::calculate_statistics( ) {

  if ( 0 == num_n )
    return 0;

  if ( 1 == num_n )
    the_statistics[ SD ] = 0.0;
  else
    the_statistics[ SD ] = sqrt( the_statistics[ SUM_T ] / ( num_n - 1 ) );

  the_statistics[ SUM2 ] = sqrt( the_statistics[ SUM2 ] );

  return num_n;

}

double RdbStats::get_value( ) {
    return input_column->getDataDouble( ); 
}

#define ARRAY_SIZE(arr)(sizeof(arr)/sizeof((arr)[0]))

void RdbStats::init( ) {

  num_n = 0;

  for ( int ii = 0; ii < ARRAY_SIZE( the_statistics ); ii++ )
    the_statistics[ ii ] = 0.0;

  the_statistics[ MAX ] = -DBL_MAX;
  the_statistics[ MIN ] = DBL_MAX;

  // purposedly leave input_column alone.

}

#undef ARRAY_SIZE

void RdbStats::normalize_results( const double norm ) {

  the_statistics[ SD ] /= norm;
  the_statistics[ MIN ] /= norm;
  the_statistics[ MAX ] /= norm;
  the_statistics[ SUM2 ] /= norm;

}

void RdbStats::set_output_columns( RDB& ordbtable ) {

    static const char* suffix[] = { "_n", "_ave", "_max", "_min", "_dev",
				      "_sum", "_rss", NULL };

    const char* colname_ptr = colname.c_str( );

    size_t counter( 0 );
    while ( suffix[ counter ] )
      ++counter;
    // + 1 to allocate an extra for the NULL sentinel.
    output_stats.reserve( counter + 1 );

    char str[ 256 ];
    sprintf( str, "%s%s", colname_ptr, suffix[ 0 ] );
    ordbtable.setColumn( str, "N" );
    output_stats[ 0 ] = ordbtable.getColumn( ordbtable.nColumns( ) - 1 );
    output_stats[ 0 ]->mapData( &num_n, 1 );
    for ( int ii = 1; ii < counter ; ii++ ) {

      int ii_1 ( ii - 1 );

      sprintf( str, "%s%s", colname_ptr, suffix[ ii ] );
      ordbtable.setColumn( str, "N" );
      output_stats[ ii_1 ] = ordbtable.getColumn( ordbtable.nColumns( ) - 1 ) ;
      output_stats[ ii_1 ]->mapData( the_statistics + ii_1, 1 );

      // Make sure that the sentinel is set to NULL
      output_stats[ ii ] = NULL;

    }
}

void RdbStats::update_statistics( )  {

    double val = get_value( );

    // only use value if it isn't NaN.  This is a simple check for NaN,
    // based upon Net Lore.

    if ( val == val ) {

	++num_n;

	if ( 1 == num_n )
	    the_statistics[ AVG ] = val;
	else {

	    //  Algorithm taken from the paper titled:
	    //    Updating Mean and Variances Estimates: An Improved Method
	    //    by D.H.D. West
	    //    Communications of the ACM
	    //    September 1979 Vol 22 Number 9
	    double tmp = ( val - the_statistics[ AVG ] );
	    the_statistics[ SUM_T ] += tmp * tmp * ( num_n - 1 ) / num_n;
	    the_statistics[ AVG ] += tmp / num_n;

	}

	the_statistics[ SUM2 ] += val * val;
	the_statistics[ SUM ] += val;
	the_statistics[ MAX ] = std::max( the_statistics[ MAX ], val );
	the_statistics[ MIN ] = std::min( the_statistics[ MIN ], val );

    }
}

std::string to_string( Normalize normalize ) {
    return 
          normalize == Normalize::Average ? "average"
        : normalize == Normalize::Median  ? "median"
        : normalize == Normalize::None    ? "none"
        :                                   "UNKNOWN";
}

std::string to_string( OverrideType override_defn ) {
    return 
          override_defn == OverrideType::Toggle ? "toggle"
        : override_defn == OverrideType::String ? "string"
        : override_defn == OverrideType::Number ? "number"
        :                                   "UNKNOWN";
}

std::string to_string( Percentile p) {
    return std::to_string( p.percentile ) + '(' + p.colname + ')';
}

std::string to_string( DefOverride d) {
    return d.colname + '(' + to_string( d.type ) + ')';
}

std::string to_string( std::string str ) { return str ; }
