#ifndef RdbStats_H
#define RdbStats_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of rdbstats
 *
 * rdbstats is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * rdbstats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <rdbxx/RDB.h>
#include <cmath>                        // sqrt fabs

#include <string>

enum struct Normalize { Average, Median, None };
enum struct OverrideType { Toggle, String, Number };

struct DefOverride {
    std::string colname;
    OverrideType type;

    DefOverride( const std::string& colname_, OverrideType type_ ) : colname(colname_), type(type_) {}

};

using DefOverrideList = std::vector<DefOverride>;

struct Percentile {
    std::string colname;
    double percentile;

    Percentile( const std::string& colname_, double percentile_ ) : colname(colname_), percentile(percentile_) {}

};
using PercentileList = std::vector<Percentile>;


/**
 * The base class to calculate :
 * average, maximum, minimum, num, stddev and sum.
 * The derived classes are: RdbStatsPercentile, RdbStatsQuartile.
 */
class RdbStats {

public:

    virtual ~RdbStats( );

    RdbStats( RDB& irdbtable, const std::string& name );

  /// Perform the final statistic for the set.
    virtual int calculate_statistics( );

  /// To get the absolute value of the average.
  double get_absolute_average( ) const { return fabs( the_statistics[AVG] ); }

    virtual void init( );

    virtual void normalize_results( const double norm );

    virtual void set_output_columns( RDB& ordbtable );
  /// Read the column from RDB++, update the statistics for the column.
    virtual void update_statistics( );

private:

  // input_column is the input rdb column ptr. It is
  // allocated by the library so do not free the memory.
  RDBColumn* input_column;

  // The pointer to the output rdb columns.
  std::vector<RDBColumn*> output_stats;

protected:

  enum Stats { AVG, MAX, MIN, SD, SUM, SUM2, SUM_T };

  double   num_n;

  std::string colname;

  double the_statistics[ 7 ];

  double get_value( );

};


std::string to_string( std::string ); // this is so ugly
std::string to_string( Normalize );
std::string to_string( OverrideType );
std::string to_string( Percentile );
std::string to_string( DefOverride );

#include <sstream>
template<typename T>
std::string to_string( std::vector<T> values ) {

    using namespace std;
    std::stringstream os;

    bool first_elem = true;
    for ( const auto& value : values ) {
        if ( ! first_elem )
            os << ", ";
        else
            first_elem = false;
        os << to_string(value);
    }

    return os.str();
}

#endif
