// File:  SelectedCols.cc

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rdbstats
//
// rdbstats is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rdbstats is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iterator>
#include <suplibxx/colselect.h>

#include "SelectedCols.h"

static void
update_column_definitions( RDB& rdbtable, const DefOverrideList& override_defn ) {

    for ( const auto& odef : override_defn ) {

        auto column = rdbtable.getColumn( odef.colname );

        std::string type;
        switch ( odef.type ) {

        case OverrideType::Toggle :
            type = RDBColumn::NUMERIC == column->getType( ) ? "S" : "N";
            break;

        case OverrideType::Number :
            type = "N";
            break;

        case OverrideType::String :
            type = "S";
            break;
        }

        column->setDef( type );
    }

}

SelectedCols::SelectedCols( RDB& rdbtable,
                            const std::vector<std::string>& include_exact,
                            const std::vector<std::string>& include_re,
                            const std::vector<std::string>& exclude_exact,
                            const std::vector<std::string>& exclude_re,
                            const std::vector<std::string>& groups,
                            const DefOverrideList& override_defn
                            ) {

  update_column_definitions( rdbtable, override_defn );

  // this is the input set of columns for the exclude/include logic
  std::vector<std::string> icolumns;

  // select all numeric, non-group columns
  {
      size_t num = rdbtable.nColumns( );
      for ( size_t ii = 0; ii < num; ii++ ) {
          std::string name = rdbtable.getColumn( ii )->getName( );
          if ( is_column_numeric( name, rdbtable ) && ! is_groupie( name, groups ) )
              icolumns.push_back( name );
      }
  }

  // if nothing is specified for include_exact, include_re, or exclude_re
  // use all of the columns
  if (
         include_exact.empty()
      && include_re.empty()
      && exclude_exact.empty()
      && exclude_re.empty()
      ) {
      selected_cols = icolumns;
      return;
  }

  // if exact columns were requested, make sure they are existent
  // numeric columns
  if ( ! include_exact.empty() ) {
      for ( const auto& name : include_exact )
          if ( ! is_column_numeric( name, rdbtable ) )
              throw Exception( name + "is not numeric" );
  }

  suplib::colselect( icolumns, include_exact, include_re, exclude_exact, exclude_re, selected_cols ); 

}

bool SelectedCols::is_column_numeric( const std::string& colname,
				      RDB& rdbtable ) {

    return RDBColumn::NUMERIC == rdbtable.getColumn( colname )->getType( );
}


bool SelectedCols::is_groupie( const std::string& name,
			       const std::vector<std::string>& groupies ) {

  std::vector< std::string >::const_iterator current_group( groupies.begin( ) ),
      end_group( groupies.end( ) );
  for ( ; current_group != end_group; ++current_group )
    if ( 0 == name.compare( *current_group ) )
      return true;

  return false;

}

void SelectedCols::print( std::ostream& os ) const {

  copy( selected_cols.begin( ), selected_cols.end( ),
	std::ostream_iterator< std::string >( os, " " ) );

}
